/*
 *  dsql.h
 *
 * 
 *  Copyright 1993 Buzz Moschetti
 * 
 *  Permission to use, copy, modify, distribute, and sell this software
 *  and its documentation for any purpose is hereby granted by Buzz
 *  Moschetti (the "Author") without fee, provided that the above
 *  copyright notice appear in all copies and that both that copyright
 *  notice and this permission notice appear in supporting documentation,
 *  and that the name of the Author. or contractions or abbreviations of
 *  the same not be used in advertising or publicity pertaining to
 *  distribution of the software without prior specific, written
 *  permission.  The Author does not make any representations about the
 *  suitability of this software for any purpose.  It is provided "as is"
 *  without express or implied warranty.
 *
 *  THE AUTHOR DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS
 *  SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND
 *  FITNESS, IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY
 *  SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER
 *  RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF
 *  CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN
 *  CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 *
 */
#if !defined(DSQL_H_INCLUDED)
#define DSQL_H_INCLUDED

/*  Make this be a pointer to the MOST restrictive datatype.  (double *)
 *  is usually it but....
 */
typedef  double *  Alignptr;


/*
 *  There are some painful diffs between the ANSI/C++ header and the
 *  standard header:
 *  Resource		gds.hxx (ANSI/C++)		gds.h (K&R)
 *  ---------------------------------------------------------------------
 *  Date datatype	GDS_QUAD			GDS_$QUAD
 *  TX handle		gds_tr_handle (typedef long)	long *
 *  DB handle		gds_db_handle			long *
 *  
 */
#if defined(__STDC__)

#if !defined(GDS_TRUE)
#include <gds.hxx>
#endif

#define  IBDATE  GDS_QUAD

#else

#if !defined(GDS_$TRUE)
#include <gds.h>
#endif

#define  IBDATE  GDS_$QUAD

/* The engine doesn't care if it is a long, long *, int, whatever -- it's
 * treated as a opaque pointer.  We might as well "upgrade" to provide the
 * following convenient datatypes:
 */
typedef long	gds_db_handle;
typedef long	gds_tr_handle;
#endif

#include <time.h>   /* For struct tm */
#include <perf.h>

typedef long	Q_db_handle;
typedef long	Q_tx_handle;


/*
 *  This is the maximum number of rows a SELECT statement can return.
 *  Bump it up if you really need more...
 */
#define SQLN  128

/*
 *  Check out q_start_trans() in dsql.c to see why we need this
 *  constant  :-(
 */
#define  MAXOPENDBS  6


/*  What q_fetch returns at end-of-cursor: */
#define  Q_EOC    -2


/*
 *  Cheezy and sick.  The enum values exactly match the datatype as defined
 *  by Interbase so we don't have to do any conversion crap:
 */
typedef enum Data_Type {
      CHAR_TYPE		= 14,
      VARY_TYPE		= 37,
      SHORT_TYPE	= 7,
      LONG_TYPE		= 8,
      FLOAT_TYPE	= 10,
      DOUBLE_TYPE	= 27,
      DATE_TYPE		= 35,
      TIME_TYPE		= 100,
      TIMESTAMP_TYPE	= 101,
      BLOB_TYPE		= -1,
      ARRAY_TYPE	= -2
} Data_Type;


typedef struct QField {
      union {
	    char *cp;
	    short s;
	    int i;
	    float flt;
	    double dbl;
	    struct tm tm;
      } v;
      Data_Type type;
      short len;
      short null;
} QField;

/*
 *  Macros to hide the internals of QField.  They demand that you pass a 
 *  QField POINTER, not the whole struct....
 */
#define  Q_FTYPE(q)	(q)->type
#define  Q_FSIZE(q)	(q)->len
#define  Q_FNULL(q)	(q)->null


#define  Q_CPV(q)	(q)->v.cp
#define  Q_SHRV(q)	(q)->v.s
#define  Q_INTV(q)	(q)->v.i
#define  Q_FLTV(q)	(q)->v.flt
#define  Q_DBLV(q)	(q)->v.dbl
#define  Q_TMV(q)	(q)->v.tm


typedef struct Query {
      SQLDA *sqlda;     /*  Communications area */

      int query_type;	/* 0 = SELECT; 1 = UPDATE, INSERT, DELETE */

      /*  Query ID is from "Q000 " to "Q999 " Yes, the trailing space is
       *  necessary for Interbase.  It is used for both the * SQL statement
       *  identifier AND the cursor identifier.  The qnum * integer is
       *  redundant now but it keeps the internal resource * allocation scheme
       *  distinct from the arbitrary string representation * (e.g. Q023) that
       *  Interbase needs.
       */
      int qnum;
      char qid[5 + 1];

      char *buffer;	/*  Raw buffer to hold fetch data */
      int buffer_len;	/*  Take a guess... */

      /*  # bytes pulled through net for each fetch.  Will usually be a little
       *  smaller than buffer_len since no extra null padding is necessary.
       */
      int transfer_len;

      int num_recs;	/* # of records fetch OR # updated */

      int cursor_status; /* -1 = N/A, 0 = closed; 1 = open */
} Query;

/*
 *  Macros to hide internals of Query:
 */
#define Q_NUMCOLS(q)	(q)->sqlda->sqld

#define Q_SELECT_TYPE   0
#define Q_EXECUTE_TYPE  1
#define Q_QTYPE(q)	(q)->query_type


int q_close_cursor();
int q_close_db();
int q_commit_trans();
int q_fetch();
int q_error_text();
int q_execute();
int q_gen();
int q_init();
int q_open_db();
int q_open_cursor();
int q_rollback_trans();
int q_start_trans();
int q_start_trans();
int q_format_fld();

#endif  /* DSQL_H_INCLUDED */
